(import glob [glob])
(import subprocess [check-output])
(import os [mkdir stat])
(import os.path [isfile :as file? isdir :as dir?])
(import templates *)
(import utils *)

(import hy.compiler [hy-eval])

(require hyrule.control [defmain])
(require hyrule.argmove [->])

(setv local-scope (locals))
(setv global-scope (globals))

(defmacro inherit [#* names] 
  (lfor name names 
    `(setx ~name (get local-scope (str (get '(~name) 0))))))

(defmacro enshrine [#* names]
  (lfor name names
    `(setv (get local-scope (str (get '(~name) 0))) ~name)))

(defn last-changed [file]
  (getattr (stat file) "st_ctime"))

(defn last-changed-prev-compile [file]
  (if (file? (-> file (.replace ".hy" ".html") (.replace "./pages/" "../site/html/")))
    (getattr (stat (-> file (.replace ".hy" ".html") (.replace "./pages/" "../site/html/"))) "st_ctime")
    False))

(defmain []
  (setv last-compile-time (last-changed "../site"))
  (setv data-dir-changed? (> (last-changed "data") last-compile-time))
  (setv assets-dir-changed? (> (last-changed "assets") last-compile-time))
  (setv last-commit-newer (> (-> (check-output #[[git log -1 --format="%at"]] :shell True) (.decode) (.strip) (int)) last-compile-time))

  (print f"last compiled: {last-compile-time :.0f}")

  (when (not (dir? "output"))
    (mkdir "output"))

  ;; only recompile pages when:
  ;;  - the source page is newer than the output
  ;;  - the `src/data` directory is newer than the output
  ;;  - the latest commit is newer than the output
  ;;  - the page is view-thought.hy and there are thoughts being recompiled
  (setv pages-to-build (lfor path (glob "./pages/**/*" :recursive True)
    :if (not-in "__pycache__" path)
    :if (or (> (last-changed path) (last-changed-prev-compile path)) data-dir-changed? assets-dir-changed? last-commit-newer)
    path))

  (when (any (map (fn [x] (in "/thoughts/" x)) pages-to-build))
    (when (all (map (fn [x] (not-in "./pages/html/view-thought.hy" x)) pages-to-build))
      (setv pages-to-build ["./pages/html/view-thought.hy" #* pages-to-build])))

  (for [path pages-to-build]
    (cond 
      (and (dir? path) (not (dir? (.replace path "pages" "output")))) 
        (do
          (print f"creating {path}")
          (mkdir (.replace path "pages" "output")))

      (file? path)
        (do 
          (print f"building {path}")
          (setv page-name (.split (cut path 2 -3) "/"))
          (enshrine page-name)
          (with [target (open (+ "./output/" (.join "/" (cut page-name 1 None)) ".html") "w")] 
            (.write target (form->html
              (hy-eval (hy.read (with [source (open path "r")]
                (.read source)))
                :locals local-scope
                :globals global-scope))))))))
